#include "ui.h"
#include "gpu.h"
#include "draw.h"

#include "arpa/inet.h"
#include "vk_mem_alloc.h"
#include "vulkan/vk_enum_string_helper.h"
#include "vulkan/vulkan_core.h"

#include "ft2build.h"
#include FT_FREETYPE_H

VkResult render_thread(GLFWwindow* window, RenderContext* render) {
  FT_Library library;

  UIContextStorage ui;

  uint32_t font_index;
  uint32_t background_index;

  VkResult result;

  VK_RESULT(create_ui_context(
        10,
        10,
        10,
        render,
        &ui));

  if(FT_Init_FreeType(&library) != FT_Err_Ok) {
    return VK_ERROR_UNKNOWN;
  }

  VK_RESULT(load_font("test.ttf", 16, VK_TRUE, library, render, &ui, &font_index));
  VK_RESULT(load_texture("background.png", render, &ui, &background_index));

  UIDrawable rects[] = {
    {
      .pos = {0.0, 0.0},
      .size = {WINDOW_WIDTH, WINDOW_HEIGHT},
      .color = {1.0, 1.0, 1.0, 1.0},
      .type = 2,
      .code = background_index,
    },
    {
      .pos = {150.0, 100.0},
      .size = {500.0, 300.0},
      .color = {1.0, 1.0, 1.0, 1.0},
    },
  };

  UILayerInput layers[] = {
    {
      .num_drawables = sizeof(rects)/sizeof(UIDrawable),
      .drawables = rects,
    },
  };

  UIContainerInput container_info = {
    .id = 0xDEADBEEF,
    .pos = {0.0, 0.0},
    .size = {WINDOW_WIDTH, WINDOW_HEIGHT},
    .layer_count = sizeof(layers)/sizeof(UILayerInput),
    .layers = layers,
  };

  VK_RESULT(create_container(&container_info, render, &ui));

  VK_RESULT(record_draw_commands(render, &ui));

  while(glfwWindowShouldClose(window) == 0) {
    glfwPollEvents();

    result = draw_frame(render);
    if(result != VK_SUCCESS) {
      fprintf(stderr, "draw_frame error: %s\n", string_VkResult(result));
      return result;
    }
  }
  return 0;
}

int logic_thread() {
  return 0;
}

int network_thread() {
  return 0;
}

int main() {
  GLFWwindow* window = init_window();
  if(window == NULL) {
    return 1;
  }

  RenderContext render = {};
  if(init_vulkan(window, &render) != VK_SUCCESS) {
    return 2;
  }

  if(render_thread(window, &render) != VK_SUCCESS) {
    return 3;
  }

  return 0;
}
